package com.me.common.ecs;

import com.badlogic.gdx.utils.Array;

final class EntityManager {

    private Engine engine;

    protected Array<Entity> entities;
    protected Array<Entity> toActivate;
    protected Array<Entity> toDeactivate;
    protected Array<Entity> toRemove;
    protected Array<Entity> removedEntities;

    public EntityManager(Engine engine) {
        this.engine = engine;

        this.entities = new Array<>();
        this.toActivate = new Array<>();
        this.toDeactivate = new Array<>();
        this.toRemove = new Array<>();
        this.removedEntities = new Array<>(false, 16);
    }

    public void update() {
        activatePending();
        deactivatePending();
        removePending();
    }

    public Entity create() {
        Entity entity;
        if (!removedEntities.isEmpty()) {
            entity = removedEntities.removeIndex(0);
            entity.reset();
        } else {
            entity = new Entity(engine);
        }
        entity.updateUniqueId();
        entities.add(entity);
        return entity;
    }

    public void remove(Entity entity) {
        toRemove.add(entity);
    }

    public void activate(Entity entity) {
        toActivate.add(entity);
    }

    public void deactivate(Entity entity) {
        toDeactivate.add(entity);
    }

    private void removePending() {
        if (toRemove.isEmpty()) {
            return;
        }

        for (Entity entity : toRemove) {
            engine.removeAllEntityComponents(entity.id);
            entities.removeValue(entity, true);
            removeEntityFromSystems(entity);
            removedEntities.add(entity);
        }

        toRemove.clear();
    }

    private void activatePending() {
        if (toActivate.isEmpty()) {
            return;
        }

        for (Entity entity : toActivate) {
            addEntityToSystems(entity);
        }
        toActivate.clear();
    }

    private void deactivatePending() {
        if (toDeactivate.isEmpty()) {
            return;
        }

        for (Entity entity : toDeactivate) {
            removeEntityFromSystems(entity);
        }
        toDeactivate.clear();
    }

    private void removeEntityFromSystems(Entity entity) {
        for (BaseSystem system : engine.systems) {
            if (!(system instanceof EntitySystem)) {
                continue;
            }

            EntitySystem entitySystem = (EntitySystem) system;
            if (entitySystem.interestedIn(entity)) {
                entitySystem.removeEntity(entity);
            }
        }
    }

    private void addEntityToSystems(Entity entity) {
        for (BaseSystem system : engine.systems) {
            if (!(system instanceof EntitySystem)) {
                continue;
            }

            EntitySystem entitySystem = (EntitySystem) system;
            if (entitySystem.interestedIn(entity)) {
                entitySystem.addEntity(entity);
            }
        }
    }


}
